'use strict';

var rxjs = require('rxjs');
var operators = require('rxjs/operators');

const seq = (s, e, step) => {
    const lst = [];
    do {
        lst.push(s);
        s += step;
    } while (s < e);
    lst.push(e);
    return lst;
};
const gen = (field) => {
    switch (field.kind) {
        case "number":
            return curr => curr === field.value;
        case "range":
            const [l, r] = field.value;
            return curr => l <= curr && r >= curr;
        case "step":
            const [range, ss] = field.value;
            const [ll, rr] = range.value;
            const step = seq(ll, rr, ss);
            return curr => step.includes(curr);
        case "list":
            const list = field.value;
            const fns = list.map(x => gen(x));
            return curr => fns.some(fn => fn(curr));
    }
};
const converter = (cron) => {
    const v = {
        minute: gen(cron.minute),
        hour: gen(cron.hour),
        date: gen(cron.date),
        month: gen(cron.month),
        week: gen(cron.week),
    };
    return ({ minute, hour, date, month, week }) => {
        return (v.minute(minute) &&
            v.hour(hour) &&
            v.date(date) &&
            v.month(month) &&
            v.week(week));
    };
};

const parseNum = (min, max, expr) => {
    const n = Number(expr);
    if (Number.isNaN(n))
        throw new Error(`expect Number, got '${expr}'`);
    if (n < min || n > max)
        throw new Error(`${n} not in range [${min}, ${max}]`);
    return n;
};
const parseRange = (min, max, expr) => {
    const range = expr.split("-");
    if (range.length === 2) {
        if (range.some(e => e.trim() === ""))
            throw new Error(`expect range, got '${expr}'`);
        const [l, r] = range;
        const ll = parseNum(min, max, l);
        const rr = parseNum(ll, max, r);
        return { kind: "range", value: [ll, rr] };
    }
    else {
        throw new Error(`expect range, got '${expr}'`);
    }
};
const parseStep = (min, max, expr) => {
    const step = expr.split("/");
    if (step.length === 2) {
        if (step.some(e => e.trim() === ""))
            throw new Error(`expect step, got '${expr}'`);
        const [f, s] = step;
        const r = f === "*"
            ? { kind: "range", value: [min, max] }
            : parseRange(min, max, f);
        const ss = parseNum(0, Infinity, s);
        return {
            kind: "step",
            value: [r, ss],
        };
    }
    else {
        throw new Error(`expect step, got '${expr}'`);
    }
};
const parseField = (min, max, expr) => {
    if (expr === "*") {
        return { kind: "range", value: [min, max] };
    }
    else if (expr.includes("/")) {
        return parseStep(min, max, expr);
    }
    else if (expr.includes("-")) {
        return parseRange(min, max, expr);
    }
    else {
        const n = parseNum(min, max, expr);
        return { kind: "number", value: n };
    }
};
const parseFields = (min, max, exprs) => {
    const l = exprs.map(x => parseField(min, max, x));
    return {
        kind: "list",
        value: l,
    };
};
const parse = (min, max, expr) => {
    const exprs = expr.split(",");
    if (exprs.length === 1) {
        return parseField(min, max, expr);
    }
    else {
        if (exprs.some(e => e.trim() === ""))
            throw new Error(`expect list, got '${expr}'`);
        return parseFields(min, max, exprs);
    }
};
const parser = (cronExpr) => {
    const exprs = cronExpr.split(/\s+/);
    if (exprs.length !== 5)
        throw new Error(`expect cron, got '${cronExpr}'`);
    const [minuteExpr, hourExpr, dateExpr, monthExpr, weekExpr] = exprs;
    const cron = {
        minute: parse(0, 59, minuteExpr),
        hour: parse(0, 23, hourExpr),
        date: parse(1, 31, dateExpr),
        month: parse(1, 12, monthExpr),
        week: parse(0, 6, weekExpr),
    };
    return cron;
};

const ONE_MINUTE = 60 * 1000;
const curr = new Date();
const next = new Date(curr.getFullYear(), curr.getMonth(), curr.getDate(), curr.getHours(), curr.getMinutes() + 1, 0, 0);
const Otimer = rxjs.timer(next, ONE_MINUTE).pipe(operators.map(() => {
    const date = new Date();
    const tdate = {
        minute: date.getMinutes(),
        hour: date.getHours(),
        date: date.getDate(),
        month: date.getMonth(),
        week: date.getDay(),
    };
    return tdate;
}));

const cron = (cronExpr) => {
    const pred = converter(parser(cronExpr));
    const obs = Otimer.pipe(operators.filter(pred));
    return obs;
};

module.exports = cron;
